/*
 * $Id: hmg-baseobject.prg 0 2011-04-17 00:00:01Z l3wonsf $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hmg-include.ch"
#include "hbqtgui.ch"
#include "hbclass.ch"
#include "common.ch"

/*==============================================================================
 HMGBASEOBJ class
 =============================================================================*/
CLASS HMGBASEOBJ FROM HMGPARENT

// internal data
   DATA lActivated                                INIT .F.
   DATA nAlignDir                                 INIT Qt_AlignAbsolute
   DATA nAlignHor                                 INIT NIL
   DATA nAlignVer                                 INIT NIL
   DATA lAutoDefault                              INIT .F.
   DATA lAutoFillBackground                       INIT .T.
   DATA cCaption                                  INIT NIL
   DATA lCreated                                  INIT .F.
   DATA nCol                                      INIT NIL
   DATA lDefault                                  INIT .F.
   DATA lFlat                                     INIT .F.
   DATA aFontColor                                INIT { 0, 0, 0, 255 }
   DATA cFontName                                 INIT NIL
   DATA nFontSize                                 INIT NIL
   DATA lFontBold                                 INIT NIL
   DATA lFontItalic                               INIT NIL
   DATA lFontUnderLine                            INIT NIL
   DATA lFontStrikeout                            INIT NIL
   DATA nFormType                                 INIT NIL
   DATA nHeight                                   INIT NIL
   DATA cIcon                                     INIT NIL
   DATA lMainForm                                 INIT NIL
   DATA bOnAfterShow                              INIT NIL
   DATA bOnInit                                   INIT NIL
   DATA bOnMouseClick                             INIT NIL
   DATA bOnRelease                                INIT NIL
   DATA oQtFont                                   INIT NIL
   DATA nRow                                      INIT NIL
   DATA cTitle                                    INIT NIL
   DATA lAutoSize                                 INIT .F.
   DATA lVisible                                  INIT .T.
   DATA nWidth                                    INIT NIL

// Methods
   METHOD Center
   METHOD FontInherit
   METHOD FontSetUp
   METHOD Hide
   METHOD Move
   METHOD Resize
   METHOD Show

// Properties
   METHOD AlignH                                  SETGET
   METHOD AlignV                                  SETGET
   METHOD AutoFillBackGround                      SETGET
   METHOD AutoSize                                SETGET
   METHOD Col                                     SETGET
   METHOD Flat                                    SETGET
   METHOD Font                                    SETGET
   METHOD FontColor                               SETGET
   METHOD FontName                                SETGET
   METHOD FontSize                                SETGET
   METHOD FontBold                                SETGET
   METHOD FontItalic                              SETGET
   METHOD FontUnderLine                           SETGET
   METHOD FontStrikeout                           SETGET
   METHOD FormType                                SETGET
   METHOD Height                                  SETGET
   METHOD IsActivated                             INLINE ::lActivated
   METHOD IsCreated                               INLINE ::lCreated
   METHOD IsMainForm                              INLINE ::lMainForm
   METHOD OnAfterShow                             SETGET
   METHOD OnInit                                  SETGET
   METHOD Row                                     SETGET
   METHOD Visible                                 SETGET
   METHOD Width                                   SETGET

ENDCLASS

/*..............................................................................
 Methods - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 FontInherit
 please use this function only within create method
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontInherit()

   IF ::lCreated == .T.
      IF VALTYPE( ::oQtFont ) == "U"
         ::oQtFont := ::QtObject:font()
      ENDIF
      IF VALTYPE( ::cFontName ) == "U"
         ::cFontName := ::oQtFont:family()
      ENDIF
      IF VALTYPE( ::nFontSize ) == "U"
         ::nFontSize := IF( ::oQtFont:pointSize()==-1, ::oQtFont:pixelSize(), ::oQtFont:pointSize() )
      ENDIF
      IF VALTYPE( ::lFontBold ) == "U"
         ::lFontBold := ::oQtFont:bold()
      ENDIF
      IF VALTYPE( ::lFontItalic ) == "U"
         ::lFontItalic := ::oQtFont:italic()
      ENDIF
      IF VALTYPE( ::lFontUnderLine ) == "U"
         ::lFontUnderLine := ::oQtFont:underline()
      ENDIF
      IF VALTYPE( ::lFontStrikeOut ) == "U"
         ::lFontStrikeOut := ::oQtFont:strikeOut()
      ENDIF
   ENDIF

RETURN NIL


/*------------------------------------------------------------------------------
 FontSetUp
 please use this function only within create method
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontSetUp()


   IF ::lCreated == .T.
      ::oQtFont:setFamily( ::cFontName )
      IF ::oQtFont:PointSize == -1
         ::oQtFont:setPixelSize( ::nFontSize )
      ELSE
         ::oQtFont:setPointSize( ::nFontSize )
      ENDIF
      ::oQtFont:setBold( ::lFontBold )
      ::oQtFont:setItalic( ::lFontItalic )
      ::oQtFont:setUnderLine( ::lFontUnderLine )
      ::oQtFont:setStrikeOut( ::lFontStrikeOut )
      ::oQTObject:SetFont( ::oQtFont )
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Hide
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Hide()

   IF ::lCreated == .T.
      ::oQTObject:Hide()
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Move
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Move()

   IF ::lCreated == .T.
      ::oQTObject:Move( ::nCol, ::nRow )
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 OnAfterShow
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:OnAfterShow( bArg1 )

   IF PCOUNT() == 0
     RETURN ::bOnAfterShow
   ELSEIF VALTYPE( bArg1 ) == "B"
      ::bOnAfterShow := bArg1
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 OnInit
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:OnInit( bArg1 )

   IF PCOUNT() == 0
     RETURN ::bOnInit
   ELSEIF VALTYPE( bArg1 ) == "B"
      ::bOnInit := bArg1
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Resize
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Resize()

   IF ::lCreated == .T.
      ::oQTObject:Resize( ::nWidth, ::nHeight )
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Show
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Show()

   IF ::lCreated == .T.
      ::oQTObject:Show()
   ENDIF

RETURN NIL

/*..............................................................................
 Properties - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 AlignH
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:AlignH( xValue )

   IF PCOUNT() == 0
      RETURN ::nAlignHor
   ELSE
      IF VALTYPE( xValue ) == "C"
         IF ALLTRIM( LOWER( xValue ) ) == "left"    ; ::nAlignHor := Qt_AlignLeft    ; ENDIF
         IF ALLTRIM( LOWER( xValue ) ) == "right"   ; ::nAlignHor := Qt_AlignRight   ; ENDIF
         IF ALLTRIM( LOWER( xValue ) ) == "center"  ; ::nAlignHor := Qt_AlignHCenter ; ENDIF
         IF ALLTRIM( LOWER( xValue ) ) == "justify" ; ::nAlignHor := Qt_AlignJustify ; ENDIF
      ELSEIF VALTYPE( xValue ) == "N"
         ::nAlignHor := IF( xValue == Qt_AlignLeft .OR. xValue == Qt_AlignRight .OR. xValue == Qt_AlignHCenter .OR. xValue == Qt_AlignJustify, xValue, ::nAlignHor )
      ENDIF
      ::SetAlign()
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 AlignV
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:AlignV( xValue )

   IF PCOUNT() == 0
      RETURN ::nAlignVer
   ELSE
      IF VALTYPE( xValue ) == "C"
         IF ALLTRIM( LOWER( xValue ) ) == "top"     ; ::nAlignVer := Qt_AlignTop     ; ENDIF
         IF ALLTRIM( LOWER( xValue ) ) == "bottom"  ; ::nAlignVer := Qt_AlignBottom  ; ENDIF
         IF ALLTRIM( LOWER( xValue ) ) == "center"  ; ::nAlignVer := Qt_AlignVCenter ; ENDIF
      ELSEIF VALTYPE( xValue ) == "N"
         ::nAlignVer := IF( xValue == Qt_AlignTop .OR. xValue == Qt_AlignBottom .OR. xValue == Qt_AlignVCenter, xValue, ::nAlignVer )
      ENDIF
      ::SetAlign()
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 AutoFillBackGround
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:AutoFillBackGround( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lAutoFillBackGround
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lAutoFillBackGround := lArg1
      IF ::lCreated == .T.
         ::oQtObject:setAutoFillBackground( ::lAutoFillBackground )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 AutoSize
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:AutoSize( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lAutoSize
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lAutoSize := lArg1
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Center
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Center()

   LOCAL nFreeW, nFreeH, oQRect

   IF ::lCreated == .T.
      oQRect := QRect( ::oParent:QtObject:screenGeometry() )
      nFreeW := oQRect:width()
      nFreeH := oQRect:height()
      ::nRow := INT( ( nFreeH - ::nHeight ) / 2 )
      ::nCol := INT( ( nFreeW - ::nWidth ) / 2 )
      ::Move()
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Col
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Col( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nCol
   ELSEIF VALTYPE( nArg1 ) == "N"
      ::nCol := nArg1
      ::nRow := IF( VALTYPE( ::nRow) == "N", ::nRow, ::oQTObject:y() )
      IF ::lCreated == .T.
         ::Move()
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Flat
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Flat( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lFlat
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lFlat := lArg1
      IF ::lCreated == .T.
         ::oQTObject:setFlat( ::lFlat )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Font
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Font( oArg1 )

   IF PCOUNT() == 0
      RETURN ::oQtFont
   ELSEIF VALTYPE( oArg1 ) == "O"
      ::oQtFont := oArg1
      IF ::lCreated == .T.
         ::cFontName := ::oQtFont:family()
         ::nFontSize := IF( ::oQtFont:pointSize()==-1, ::oQtFont:pixelSize(), ::oQtFont:pointSize() )
         ::lFontBold := ::oQtFont:bold()
         ::lFontItalic := ::oQtFont:italic()
         ::lFontUnderLine := ::oQtFont:strikeOut()
         ::lFontStrikeOut := ::oQtFont:underline()
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontColor
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontColor( aArg1 )

   LOCAL oPalette, oQtFontColor, oQtFontBrush

   IF PCOUNT() == 0
      RETURN ::aFontColor
   ELSEIF VALTYPE( aArg1 ) == "A"
      ::aFontColor := IF( LEN( aArg1 ) == 4, aArg1, { aArg1[1], aArg1[2], aArg1[3], 255} )
      IF ::lCreated == .T.
         oQtFontColor := QColor( ::aFontColor[1], ::aFontColor[2], ::aFontColor[3], ::aFontColor[4] )
         oQtFontBrush := QBrush( oQtFontColor, Qt_SolidPattern )
         oPalette := ::oQTObject:palette()
	      oPalette:setBrush( QPalette_Active, QPalette_WindowText, oQtFontBrush)
         oPalette:setBrush( QPalette_Inactive, QPalette_WindowText, oQtFontBrush)
	      ::oQtObject:setPalette( oPalette )
	   ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontName
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontName( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cFontName
   ELSEIF VALTYPE( cArg1 ) == "C"
      ::cFontName := cArg1
      IF ::lCreated == .T.
         ::oQtFont:SetFamily( ::cFontName )
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontSize
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontSize( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nFontSize
   ELSEIF VALTYPE( nArg1 ) == "N"
      ::nFontSize := nArg1
      IF ::lCreated == .T.
         IF ::oQtFont:PointSize() == -1
            ::oQtFont:setPixelSize( ::nFontSize )
         ELSE
            ::oQtFont:setPointSize( ::nFontSize )
         ENDIF
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontBold
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontBold( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lFontBold
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lFontBold := lArg1
      IF ::lCreated == .T.
         ::oQtFont:SetBold( ::lFontBold )
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontItalic
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontItalic( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lFontItalic
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lFontItalic := lArg1
      IF ::lCreated == .T.
         ::oQtFont:SetItalic( ::lFontItalic )
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontUnderLine
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontUnderLine( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lFontUnderLine
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lFontUnderLine := lArg1
      IF ::lCreated == .T.
         ::oQtFont:SetUnderLine( ::lFontUnderline )
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FontStrikeOut
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FontStrikeOut( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lFontStrikeOut
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lFontStrikeOut := lArg1
      IF ::lCreated == .T.
         ::oQtFont:SetStrikeOut( ::lFontStrikeout )
         ::oQTObject:SetFont( ::oQtFont )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 FormType
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:FormType( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nFormType
   ELSEIF VALTYPE( nArg1 ) == "N" .AND. nArg1 >= WND_STANDARD .AND. nArg1 <= WND_SPLITCHILD
      ::nFormType := nArg1
      ::lMainForm := IF( ::nFormType == WND_MAIN, .T., .F. )
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Height
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Height( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nHeight
   ELSEIF VALTYPE( nArg1 ) == "N"
      ::nHeight := nArg1
      IF ::lCreated == .T.
         ::nWidth := IF( VALTYPE( ::nWidth ) == "N", ::nWidth, ::oQTObject:Width() )
         ::Resize()
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Row
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Row( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nRow
   ELSEIF VALTYPE( nArg1 ) == "N"
      ::nRow := nArg1
      IF ::lCreated == .T.
         ::nCol := IF( VALTYPE( ::nCol) == "N", ::nCol, ::oQTObject:x() )
         ::Move()
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Visible
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Visible( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lVisible
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lVisible := lArg1
      IF ::lCreated == .T.
         IF ::lVisible == .T.
            ::Show()
         ELSE
            ::Hide()
         ENDIF
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Width
 -----------------------------------------------------------------------------*/
METHOD HMGBASEOBJ:Width( nArg1 )

   IF PCOUNT() == 0
      RETURN ::nWidth
   ELSEIF VALTYPE( nArg1 ) == "N"
      ::nWidth := nArg1
      IF ::lCreated == .T.
         ::nHeight := IF( VALTYPE( ::nHeight ) == "N", ::nHeight, ::oQTObject:Height() )
         ::oQTObject:resize( ::nWidth, ::nHeight )
      ENDIF
   ENDIF

RETURN NIL
