/*
 * $Id: hmg-button.prg 0 2011-04-17 00:00:01Z l3wonsf $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hmg-include.ch"
#include "hbqtgui.ch"
#include "hbclass.ch"
#include "common.ch"

/*==============================================================================
 HMGBUTTON class
 =============================================================================*/
CLASS HMGBUTTON FROM HMGSHARED, HMGBASEOBJ


// Methods
   METHOD New
   METHOD Create

// Properties
   METHOD AutoDefault                             SETGET
   METHOD Caption                                 SETGET
   METHOD Default                                 SETGET
   METHOD OnMouseClick                            SETGET
   METHOD Icon                                    SETGET

ENDCLASS

/*..............................................................................
 Methods - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 New
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:New( cName, oParent )

// TO prevent use without QT starting
   IF ::lQtStarted == .F.
      HMGAPPLICATION():Start()
   ENDIF

   IF ::lHmgError == .T.
      ::sQtApplication:Exit()
      ::sQtApplication:Quit()
// TODO HmgStdErr LOG
      QUIT
   ENDIF

// set-up a default name
   ::cName          := IF( VALTYPE( cName) == "C", cName, '_HMG_OBJECT_' + hb_ntos( ::nObjectCounter++ ) )

// set-up parent in this order: parameter, current or default (last one is Qt QApplication():DeskTop()
   ::oParent := IF( hb_IsObject( oParent ), oParent, ::GetParent() )

// add HmgObject to its parent
   ::oParent:AddChild( Self )
   ::oParent:AddData( ::cName , Self )

// create QT object with its parent
   ::oQtObject := QPushButton( ::oParent:Qtobject() )

RETURN Self

/*------------------------------------------------------------------------------
 Create
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:Create()

   LOCAL oQtFontMetrics, nWidth, nHeight

// TO prevent more time creation
   IF ::lCreated == .T.
      RETURN NIL
   ENDIF

// QtObject created
   ::lCreated := .T.

// font
   ::FontInherit()
   ::FontSetUp()
   ::FontColor( ::aFontColor )

   ::AutoFillBackground( ::lAutoFillBackground )

// alignment
   ::nAlignHor := IF( VALTYPE( ::nAlignHor ) == "U", Qt_AlignLeft, ::nAlignHor )
   ::nAlignVer := IF( VALTYPE( ::nAlignVer ) == "U", Qt_AlignVCenter, ::nAlignVer )

// autodefault
   ::AutoDefault( ::lAutoDefault )

// flat
   ::Flat( ::lFlat )

// default
   ::Default( ::lDefault )

// caption
   ::cCaption := IF( VALTYPE( ::cCaption ) == "C", ::cCaption, "" )
   ::Caption( ::cCaption )

// Icon
   ::Icon( ::cIcon )

// size
   oQtFontMetrics := QFontMetrics( ::oQtFont )
   nWidth := oQtFontMetrics:width( ::oQTObject:text() )
   nHeight := oQtFontMetrics:height( ::oQTObject:text() )
   IF VALTYPE( ::lAutoSize ) == "L" .AND. ::lAutoSize == .T.
      ::nWidth := nWidth
      ::nHeight := nHeight
   ELSE
      ::nWidth := IF( VALTYPE( ::nWidth ) == "N", ::nWidth, nWidth )
      ::nHeight := IF( VALTYPE( ::nHeight ) == "N", ::nHeight, nHeight )
   ENDIF
   ::Resize()

// move
   ::nRow := IF( VALTYPE( ::nRow) == "N", ::nRow, 0 )
   ::nCol := IF( VALTYPE( ::nCol) == "N", ::nCol, 0 )
   ::Move()

// connecting default event
   ::OnMouseClick( ::bOnMouseClick )

// Show QtObject if visible
   IF ::lVisible == .T.
      ::oQTObject:Show()
   ELSE
      ::oQTObject:Hide()
   ENDIF

RETURN NIL

/*..............................................................................
 Properties - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 AutoDefault
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:AutoDefault( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lAutoDefault
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lAutoDefault := lArg1
      IF ::lCreated == .T.
         ::oQTObject:setAutoDefault( ::lAutoDefault )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Caption
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:Caption( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cCaption
   ELSEIF VALTYPE( cArg1 ) == "C"
      ::cCaption := cArg1
      IF ::lCreated == .T.
         ::oQTObject:setText( ::cCaption )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Default
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:Default( lArg1 )

   IF PCOUNT() == 0
      RETURN ::lDefault
   ELSEIF VALTYPE( lArg1 ) == "L"
      ::lDefault := lArg1
      IF ::lCreated == .T.
         ::oQTObject:setDefault( ::lDefault )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 OnMouseClick
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:OnMouseClick( bArg1 )

   IF PCOUNT() == 0
     RETURN ::bOnMouseClick
   ELSEIF VALTYPE( bArg1 ) == "B"
      ::bOnMouseClick := bArg1
      IF ::lCreated == .T.
         ::oQTObject:Connect( "clicked()" , ::bOnMouseClick )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Icon
 -----------------------------------------------------------------------------*/
METHOD HMGBUTTON:Icon( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cIcon
   ELSEIF VALTYPE( cArg1 ) == "C"
      ::cIcon := cArg1
      IF ::lCreated == .T.
         ::oQTObject:SetIcon( cArg1 )
      ENDIF
   ENDIF

RETURN NIL

