/*
 * $Id: hmg-common.prg 0 2011-04-17 00:00:01Z l3wonsf $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"

/*==============================================================================
 HMGPARENT class
 =============================================================================*/
CLASS HMGPARENT

// common internal data
   DATA oQtObject                                 INIT NIL                 // Qtobject
   DATA aChildren                                 INIT {}                  // children (form, label, etc.)
   DATA cName                                     INIT NIL                 // Hmg Object name
   DATA oParent                                   INIT NIL                 // Hmg parent Object

// Methods
   METHOD AddChild
   METHOD AddData
   METHOD AddParent
   METHOD CreatePendingChildControls
   METHOD GetParent
   METHOD RemoveParent

// Properties
   METHOD Name                                    SETGET
   METHOD Parent                                  SETGET
   METHOD QtObject                                SETGET

ENDCLASS

/*..............................................................................
 Methods - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 AddChild
 ------------------------------------------------------------------------------*/
METHOD HMGPARENT:AddChild( oObject )

   AADD( ::aChildren, oObject )

RETURN NIL

/*------------------------------------------------------------------------------
 AddData
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:AddData( cObjName, oObject )

   __objAddData( Self, cObjName )

   ::&cObjName := oObject

RETURN NIL

/*------------------------------------------------------------------------------
 AddParent
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:AddParent()

   AADD( ::aParentLevel, Self )

RETURN NIL

/*------------------------------------------------------------------------------
 CreatePendingChildControls
 ------------------------------------------------------------------------------*/
METHOD HMGPARENT:CreatePendingChildControls()

   LOCAL nI

   FOR nI := 1 TO LEN( ::aChildren )
      IF __objHasMethod( ::aChildren[nI], "create" ) == .T.
         ::aChildren[nI]:Create()
      ENDIF
   NEXT nI

RETURN NIL

/*------------------------------------------------------------------------------
 GetParent
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:GetParent()

   LOCAL oParent, nX := LEN( ::aParentLevel )
   
   oParent := IF( nX == 0, ::oSuperParent, ::aParentLevel[nX] )

RETURN oParent

/*------------------------------------------------------------------------------
 RemoveParent
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:RemoveParent()

   IF LEN( ::aParentLevel ) > 0
      ::aParentLevel := ASIZE( ::aParentLevel, LEN( ::aParentLevel ) -1 )
   ENDIF

RETURN NIL

/*..............................................................................
 Properties - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 Name
 ATTENTION: it's better assign value in other place
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:Name( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cName
   ELSEIF hb_IsChar( cArg1 )
      ::cName := cArg1
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 QtObject
 ATTENTION: it's better assign value in other place
 -----------------------------------------------------------------------------*/
METHOD HMGPARENT:QtObject( oArg1 )

   IF PCOUNT() == 0
      RETURN ::oQtObject
   ELSEIF hb_IsObject( oArg1 )
      ::oQtObject := oArg1
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Parent
 ATTENTION: it's better assign value in other place
 ------------------------------------------------------------------------------*/
METHOD HMGPARENT:Parent( oArg1 )

   IF PCOUNT() == 0
      RETURN ::oParent
   ELSEIF hb_IsObject( oArg1 )
      ::oParent := oArg1
   ENDIF

RETURN NIL
