/*
 * $Id: hmg-window.prg 0 2011-04-17 00:00:01Z l3wonsf $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hmg-include.ch"
#include "hbqtgui.ch"
#include "hbclass.ch"
#include "common.ch"

/*==============================================================================
 HMGWINDOW class
 =============================================================================*/
CLASS HMGWINDOW FROM HMGSHARED, HMGBASEOBJ

// internal data

// Methods
   METHOD New
   METHOD Create
   METHOD Activate
   METHOD EndWindow
   METHOD Release

// Properties
   METHOD Icon                                    SETGET
   METHOD OnRelease                               SETGET
   METHOD Title                                   SETGET

ENDCLASS

/*..............................................................................
 Methods - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 New
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:New( cName, oParent )

// TO prevent use without QT starting
   IF ::lQtStarted == .F.
      HMGAPPLICATION():Start()
   ENDIF

   IF ::lHmgError == .T.
      ::sQtApplication:Exit()
      ::sQtApplication:Quit()
// TODO HmgStdErr LOG
      QUIT
   ENDIF

// set-up a default name
   ::cName          := IF( VALTYPE( cName) == "C", cName, '_HMG_OBJECT_' + hb_ntos( ::nObjectCounter++ ) )

// set-up parent in this order: parameter, current, default (last one is Qt QApplication():DeskTop()
   ::oParent := IF( hb_IsObject( oParent ), oParent, ::GetParent() )

// now form it's the current parent
   ::AddParent()

// add HmgObject to its parent
   ::oParent:AddChild( Self )
   ::oParent:AddData( ::cName , Self )

// create QT object with its parent
   ::oQtObject := QMainWindow( ::oParent:QtObject )

RETURN Self

/*------------------------------------------------------------------------------
 Create
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:Create()

   LOCAL oQrect, nWidth, nHeight

// TO prevent more time creation
   IF ::lCreated == .T.
      RETURN NIL
   ENDIF

// QtObject created
   ::lCreated := .T.

// from here we must setup unsetting var for the object: most of them are NIL
// font
   ::FontInherit()
   ::FontSetUp()

// title
   ::cTitle := IF( VALTYPE( ::cTitle ) == "C", ::cTitle, "" )
   ::Title( ::cTitle )

// icon
   ::Icon( ::cIcon )

// size
   oQRect := QRect( ::oParent:QtObject:availableGeometry() )
   nWidth := oQRect:width()
   nHeight := oQRect:width()
   IF VALTYPE( ::lAutoSize ) == "L" .AND. ::lAutoSize == .T.
      ::nWidth := nWidth
      ::nHeight := nHeight
   ELSE
      ::nWidth := IF( VALTYPE( ::nWidth ) == "N", ::nWidth, nWidth )
      ::nHeight := IF( VALTYPE( ::nHeight ) == "N", ::nHeight, nHeight )
   ENDIF
   ::Resize()

// move
   ::nRow := IF( VALTYPE( ::nRow) == "N", ::nRow, 0 )
   ::nCol := IF( VALTYPE( ::nCol) == "N", ::nCol, 0 )
   ::Move()

// connecting default event
   IF ::nFormType == WND_MAIN
      IF VALTYPE( ::bOnRelease ) == "B"
         ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease ), ::sQtApplication:Quit() } )
      ELSE
         ::oQTObject:connect( QEvent_Close , {|| ::sQtApplication:Quit() } )
      ENDIF
   ELSEIF ::nFormType == WND_MODAL
      IF VALTYPE( ::bOnRelease ) == "B"
         ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease ), ::oQTObject:Exit(), ::oQTObject:Quit() } )
      ELSE
         ::oQTObject:connect( QEvent_Close , {|| ::oQTObject:Exit(), ::oQTObject:Quit() } )
      ENDIF
   ELSE
      IF VALTYPE( ::bOnRelease ) == "B"
         ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease )} )
      ENDIF
   ENDIF

// create child controls
   ::CreatePendingChildControls()

RETURN NIL

/*------------------------------------------------------------------------------
 Activate
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:Activate()

// Is it already created?
   IF ::lCreated == .F.
      ::Create()
   ENDIF

// Is it already activated?
   IF ::lActivated == .T.
      RETURN NIL
   ENDIF
   ::lActivated := .T.

   IF VALTYPE( ::bOnInit ) == "B"
      EVAL( ::bOnInit )
   ENDIF

// Show QtObject
   ::oQTObject:Show()

   IF VALTYPE( ::bOnAfterShow ) == "B"
      EVAL( ::bOnAfterShow )
   ENDIF

// Start Qt handling
   IF ::nFormType == WND_MAIN
      ::sQtApplication:Exec()
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 EndWindow
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:EndWindow()

   ::RemoveParent()

RETURN NIL

/*------------------------------------------------------------------------------
 Release
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:Release()

   ::oQtObject:close()

RETURN NIL

/*..............................................................................
 Properties - section
 .............................................................................*/

/*------------------------------------------------------------------------------
 Icon (windowIcon)
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:Icon( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cIcon
   ELSEIF VALTYPE( cArg1 ) == "C"
      ::cIcon := cArg1
      IF ::lCreated == .T.
         ::oQTObject:setWindowIcon( ::cIcon )
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 OnRelease
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:OnRelease( bArg1 )

   IF PCOUNT() == 0
     RETURN ::bOnRelease
   ELSEIF VALTYPE( bArg1 ) == "B"
      ::bOnRelease := bArg1
      IF ::lCreated == .T.
         IF ::nFormType == WND_MAIN
            IF VALTYPE( ::bOnRelease ) == "B"
               ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease ), ::sQtApplication:Quit() } )
            ELSE
               ::oQTObject:connect( QEvent_Close , {|| ::sQtApplication:Quit() } )
            ENDIF
         ELSEIF ::nFormType == WND_MODAL
            IF VALTYPE( ::bOnRelease ) == "B"
               ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease ), ::oQTObject:Exit(), ::oQTObject:Quit() } )
            ELSE
               ::oQTObject:connect( QEvent_Close , {|| ::oQTObject:Exit(), ::oQTObject:Quit() } )
            ENDIF
         ELSE
            IF VALTYPE( ::bOnRelease ) == "B"
               ::oQTObject:connect( QEvent_Close , {|| EVAL( ::bOnRelease )} )
            ENDIF
         ENDIF
      ENDIF
   ENDIF

RETURN NIL

/*------------------------------------------------------------------------------
 Title (windowTitle)
 -----------------------------------------------------------------------------*/
METHOD HMGWINDOW:Title( cArg1 )

   IF PCOUNT() == 0
      RETURN ::cTitle
   ELSEIF VALTYPE( cArg1 ) == "C"
      ::cTitle := cArg1
      IF ::lCreated == .T.
         ::oQtObject:setWindowTitle( ::cTitle )
      ENDIF
   ENDIF

RETURN NIL
